@@ -459,14 +459,6 @@ process_cups_notification (GsdPrintNotificationsManager *manager,
                         secondary_text = g_strdup (printer_name);
                 }
         } else if (g_strcmp0 (notify_subscribed_event, "printer-deleted") == 0) {
-                if (is_local_dest (printer_name,
-                                   manager->priv->dests,
-                                   manager->priv->num_dests)) {
-                        /* Translators: A printer has been removed */
-                        primary_text = g_strdup (_("Printer removed"));
-                        secondary_text = g_strdup (printer_name);
-                }
-
                 cupsFreeDests (manager->priv->num_dests, manager->priv->dests);
                 manager->priv->num_dests = cupsGetDests (&manager->priv->dests);
         } else if (g_strcmp0 (notify_subscribed_event, "job-completed") == 0 && my_job) {
