@@ -168,7 +168,8 @@ get_device_id (GsdWacomDevice *device)
 	GdkDevice *gdk_device;
 	int id;
 
-	g_object_get (device, "gdk-device", &gdk_device, NULL);
+	gdk_device = gsd_wacom_device_get_gdk_device (device);
+
 	if (gdk_device == NULL)
 		return -1;
 	g_object_get (gdk_device, "device-id", &id, NULL);
@@ -476,11 +477,11 @@ set_keep_aspect (GsdWacomDevice *device,
 	/* Get corresponding monitor size */
 	mapper = gsd_device_mapper_get ();
 	device_manager = gsd_device_manager_get ();
-	g_object_get (device, "gdk-device", &gdk_device, NULL);
+
+	gdk_device = gsd_wacom_device_get_gdk_device (device);
 	gsd_device = gsd_x11_device_manager_lookup_gdk_device (GSD_X11_DEVICE_MANAGER (device_manager),
 							       gdk_device);
 	monitor = gsd_device_mapper_get_device_monitor (mapper, gsd_device);
-	g_object_unref (gdk_device);
 
 	/* Adjust area to match the monitor aspect ratio */
 	g_debug ("Initial device area: (%d,%d) (%d,%d)", area[0], area[1], area[2], area[3]);
@@ -1340,10 +1341,9 @@ switch_monitor (GsdWacomManager *manager,
 	if (n_monitors < 2)
 		return;
 
-        g_object_get (device, "gdk-device", &gdk_device, NULL);
+        gdk_device = gsd_wacom_device_get_gdk_device (device);
         gsd_device = gsd_x11_device_manager_lookup_gdk_device (GSD_X11_DEVICE_MANAGER (gsd_device_manager_get ()),
                                                                gdk_device);
-        g_object_unref (gdk_device);
         current_monitor =
                 gsd_device_mapper_get_device_monitor (manager->priv->device_mapper,
                                                       gsd_device);
@@ -1367,7 +1367,7 @@ notify_osd_for_device (GsdWacomManager *manager,
         GdkScreen *screen;
         gint monitor_num;
 
-        g_object_get (device, "gdk-device", &gdk_device, NULL);
+        gdk_device = gsd_wacom_device_get_gdk_device (device);
         gsd_device = gsd_x11_device_manager_lookup_gdk_device (GSD_X11_DEVICE_MANAGER (gsd_device_manager_get ()),
                                                                gdk_device);
         monitor_num = gsd_device_mapper_get_device_monitor (manager->priv->device_mapper,
@@ -1598,11 +1598,10 @@ check_need_for_calibration (GsdWacomDevice  *device)
 
         screen = gdk_screen_get_default ();
         mapper = gsd_device_mapper_get ();
-        g_object_get (device, "gdk-device", &gdk_device, NULL);
+        gdk_device = gsd_wacom_device_get_gdk_device (device);
         gsd_device = gsd_x11_device_manager_lookup_gdk_device (GSD_X11_DEVICE_MANAGER (gsd_device_manager_get ()),
                                                                gdk_device);
         monitor = gsd_device_mapper_get_device_monitor (mapper, gsd_device);
-        g_object_unref (gdk_device);
 
 	if (monitor < 0) {
 		geometry.width = gdk_screen_get_width (screen);
@@ -1964,10 +1963,9 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
                         if (type == WACOM_TYPE_PAD) {
                                 GdkDevice *gdk_device;
 
-                                g_object_get (device, "gdk-device", &gdk_device, NULL);
+                                gdk_device = gsd_wacom_device_get_gdk_device (device);
                                 grab_button (gdk_x11_device_get_id (gdk_device),
                                              FALSE, manager->priv->screen);
-                                g_object_unref (gdk_device);
                         }
                 }
 
