@@ -404,6 +404,7 @@ gsd_wacom_osd_button_set_active (GsdWacomOSDButton *osd_button,
 		osd_button->priv->timeout_id = g_timeout_add (BUTTON_TIMER_STEP,
 							      (GSourceFunc) gsd_wacom_osd_button_timer,
 							      osd_button);
+		g_source_set_name_by_id (osd_button->priv->timeout_id, "[gnome-settings-daemon] gsd_wacom_osd_button_timer");
 	} else if (osd_button->priv->next_state) {
 		/* it was on and now should be on again */
 		if (previous_state == active) {
@@ -1447,6 +1448,7 @@ gsd_wacom_osd_window_motion_notify_event (GtkWidget      *widget,
 	window->priv->cursor_timeout = g_timeout_add_seconds (CURSOR_HIDE_TIMEOUT,
 							      cursor_timeout_source_func,
 							      window);
+	g_source_set_name_by_id (window->priv->cursor_timeout, "[gnome-settings-daemon] cursor_timeout_source_func");
 
 	return FALSE;
 }
@@ -1626,20 +1628,16 @@ gsd_wacom_osd_window_realized (GtkWidget *widget,
 
 	/* Determine the monitor for that device and set appropriate fullscreen mode*/
 	monitor = gsd_wacom_device_get_display_monitor (osd_window->priv->pad);
+
 	if (monitor == GSD_WACOM_SET_ALL_MONITORS) {
-		/* Covers the entire screen */
-		osd_window->priv->screen_area.x = 0;
-		osd_window->priv->screen_area.y = 0;
-		osd_window->priv->screen_area.width = gdk_screen_get_width (screen);
-		osd_window->priv->screen_area.height = gdk_screen_get_height (screen);
-		gdk_screen_get_monitor_geometry (screen, 0, &osd_window->priv->monitor_area);
-		gdk_window_set_fullscreen_mode (gdk_window, GDK_FULLSCREEN_ON_ALL_MONITORS);
-	} else {
-		gdk_screen_get_monitor_geometry (screen, monitor, &osd_window->priv->screen_area);
-		osd_window->priv->monitor_area = osd_window->priv->screen_area;
-		gdk_window_set_fullscreen_mode (gdk_window, GDK_FULLSCREEN_ON_CURRENT_MONITOR);
+		/* Pick a monitor for the OSD to appear */
+		monitor = gdk_screen_get_primary_monitor (screen);
 	}
 
+	gdk_screen_get_monitor_geometry (screen, monitor, &osd_window->priv->screen_area);
+	osd_window->priv->monitor_area = osd_window->priv->screen_area;
+	gdk_window_set_fullscreen_mode (gdk_window, GDK_FULLSCREEN_ON_CURRENT_MONITOR);
+
 	gtk_window_set_default_size (GTK_WINDOW (osd_window),
 	                             osd_window->priv->screen_area.width,
 	                             osd_window->priv->screen_area.height);
